#ifndef CIMGMOREPAT_H
#define CIMGMOREPAT_H

#include <iostream>
#include <string> 
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <sys/file.h>
#include <fcntl.h>
#include <unistd.h>
#include "CImg.h"

#define T_Line_Max 1244
#define T_Col_Max  1244
#define ABSOLUE(x) ((x)>(0)?(x):(-x))   
#define bound(n,m)  ((n < 0) ? (0) : ((n > m-1) ? (m-1) : (n)))

using namespace cimg_library;
using namespace std;

CImg<double> gradXCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GX(3,1);
		filtre_GX(0,0)=-1; filtre_GX(2,0)=1;
							
	return im.get_convolve(filtre_GX,1,false)/2.;		
	
}
CImg<double> gradYCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GY(1,3);
		filtre_GY(0,0)=-1; filtre_GY(0,2)=1;
							
	return im.get_convolve(filtre_GY,1,false)/2.;		

}
CImg<double> gradXXCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GX(3,1);
		filtre_GX(0,0)=1; filtre_GX(1,0)=-2;filtre_GX(2,0)=1;
							
	return im.get_convolve(filtre_GX,1,false);		
}
CImg<double> gradYYCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GY(1,3);
		filtre_GY(0,0)=1; filtre_GY(0,1)=-2;filtre_GY(0,2)=1;
							
	return im.get_convolve(filtre_GY,1,false);		

}
CImg<double> gradXYCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GXY(3,3);
		filtre_GXY(0,0)=1; filtre_GXY(0,1)=0;filtre_GXY(0,2)=-1;
		filtre_GXY(1,0)=0; filtre_GXY(1,1)=0;filtre_GXY(1,2)=0;
		filtre_GXY(2,0)=-1; filtre_GXY(2,1)=0;filtre_GXY(2,2)=1;
							
	return im.get_convolve(filtre_GXY,1,false)/4.;		

}
CImg<double> gradXXXCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GX(5,1);
		filtre_GX(0,0)=-1; filtre_GX(1,0)=2;filtre_GX(2,0)=0;filtre_GX(3,0)=-2;filtre_GX(4,0)=1;				
	return im.get_convolve(filtre_GX,1,false)/2.;		
}
CImg<double> gradYYYCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GY(1,5);
		filtre_GY(0,0)=-1; filtre_GY(0,1)=2;filtre_GY(0,2)=0;filtre_GY(0,3)=-2;filtre_GY(0,4)=1;						
	return im.get_convolve(filtre_GY,1,false)/2.;		
}
CImg<double> gradXXYCenter2nd(const CImg<double> &im){
	
	CImg<float> filtre_GX(3,3);
		filtre_GX(0,0)=-1; filtre_GX(0,1)=0;  filtre_GX(0,2)=1;
		filtre_GX(1,0)=2;  filtre_GX(1,1)=0;  filtre_GX(1,2)=-2;
		filtre_GX(2,0)=-1;  filtre_GX(2,1)=0; filtre_GX(2,2)=1;
														
	return im.get_convolve(filtre_GX,1,false)/2.;		

}

CImg<double> gradYYXCenter2nd(const CImg<double> &im){

	CImg<float> filtre_GY(3,3);
		filtre_GY(0,0)=-1; filtre_GY(0,1)=2;filtre_GY(0,2)=-1;
		filtre_GY(1,0)=0; filtre_GY(1,1)=0;filtre_GY(1,2)=0;
		filtre_GY(2,0)=1; filtre_GY(2,1)=-2;filtre_GY(2,2)=1;

	return im.get_convolve(filtre_GY,1,false)/2.;		

}

/*Fills one of the channels of a multiband image with a monoband image*/

CImg<float> subsample(const  CImg<float> & im, int factor, int ofx, int ofy){
	
	int dx=im.dimx();
	int dy=im.dimy();
	int dxSub=(int)dx/factor;
	int dySub=(int)dy/factor;
	CImg<float> imSub(dxSub,dySub);
	imSub.fill(0);			
	for (int i=ofx;i<dx;i+=factor){
		for (int j=ofy;j<dy;j+=factor){
			imSub((int)((float)(i-ofx)/(float)factor),(int)((float)(j-ofy)/(float)factor))=im(i,j);
		}
	}
	imSub=imSub.crop(0,0,dxSub-((int)(ofx/factor))*2-1,dySub-((int)(ofy/factor))*2-1);
	return imSub;
}
CImg<float>  Filtre_2D(const CImg<float>& b, const CImg<float>& filtre){
    
    /*Filter with dirichlet boundary condition*/
    int nb_ligF=filtre.dimy(), nb_colF=filtre.dimx();
    int nb_line=b.dimy(), nb_col=b.dimx();
    int n= nb_colF/2;
    int m= nb_ligF/2;
    CImg<float> a(nb_col,nb_line);
    
     for (int i=0;i<nb_line;i++)
    {
	for (int j=0;j<nb_col;j++)
	{
	    a(j,i)=0.0;

	    for (int k=-m;k<m+1;k++)
	    for (int l=-n;l<n+1;l++)
	    {
		a(j,i)+= b[bound(j+l,nb_col)+bound(i+k,nb_line)*nb_col] * filtre[(k+m)*nb_colF+(l+n)];

	    }
	}
    }
    return a;
}

CImg<float> subsampleSparse_SquareBlur(const  CImg<float> & im, int factor, int ofx, int ofy){
	
	int dx=im.dimx();
	int dy=im.dimy();
	int dxSub=(int)dx/factor;
	int dySub=(int)dy/factor;
	CImg<float> imSub(dxSub,dySub);	
	CImg<float> imTmp(dx,dy);	
	CImg<float> imTmp2(dx,dy);		
	CImg<float> filtre(factor+1,factor+1);filtre.fill(1.);
	CImgStats stats(im,false);
	for (int i=ofx;i<dx;i++){for (int j=ofy;j<dy;j++){if(im(j,i)>=stats.max-1){im(j,i)=0;imTmp(j,i)=0;}else{imTmp(j,i)=1;}}}
	imTmp=Filtre_2D(imTmp,filtre);
	imTmp2=Filtre_2D(im,filtre);
	for (int i=ofx;i<dx;i++){for (int j=ofy;j<dy;j++){if (imTmp(j,i)!=0){im(j,i)=imTmp2(j,i)/imTmp(j,i);}else{im(j,i)=0;}}}
	for (int i=ofx;i<dx;i+=factor){
		for (int j=ofy;j<dy;j+=factor){
			imSub((int)((float)(j-ofx)/(float)factor),(int)((float)(i-ofy)/(float)factor))=im(j,i);
		}
	}
	imSub=imSub.crop(0,0,dxSub-((int)(ofx/factor))*2-1,dySub-((int)(ofy/factor))*2-1);
	return imSub;
}
 

CImg<float> subsampleSparse(const  CImg<float> & im,  int ofx, int ofy){

	int factor=2;
	
 	CImg<float> filtre_PB_H(5,1); 	/*-- filtre passe-bas horizontal (burt) --*/
	filtre_PB_H(0,0)=0.05;
	filtre_PB_H(1,0)=0.25;
	filtre_PB_H(2,0)=0.45;
	filtre_PB_H(3,0)=0.25;
	filtre_PB_H(4,0)=0.05;
	CImg<float> filtre_PB_V(1,5);	/*-- filtre passe-bas vertical (burt) --*/	
	filtre_PB_V(0,1)=0.05;
	filtre_PB_V(0,1)=0.25;
	filtre_PB_V(0,2)=0.45;
	filtre_PB_V(0,3)=0.25;
	filtre_PB_V(0,4)=0.05;

	int dx=im.dimx();
	int dy=im.dimy();
	int dxSub=(int)dx/factor;
	int dySub=(int)dy/factor;
	CImg<float> imSub(dxSub,dySub);	
	CImg<float> imTmp(dx,dy);	
	CImg<float> imTmp2(dx,dy);		


	for (int i=ofx;i<dx;i++){for (int j=ofy;j<dy;j++){if(im(j,i)>=1022||im(j,i)==0){im(j,i)=0;imTmp(j,i)=0;}else{imTmp(j,i)=1;}}}

	imTmp.correlate(filtre_PB_H);
	imTmp2=im.get_correlate(filtre_PB_H);		
	for (int i=ofx;i<dx;i++){for (int j=ofy;j<dy;j++){if (imTmp(j,i)!=0){im(j,i)=imTmp2(j,i)/(imTmp(j,i));}else{im(j,i)=0;}}}

	
	for (int i=ofx;i<dx;i++){for (int j=ofy;j<dy;j++){if (im(j,i)!=0){imTmp(j,i)=1;}else{imTmp(j,i)=0;}}}
	imTmp.correlate(filtre_PB_V);
	imTmp2=im.get_correlate(filtre_PB_V);		

	for (int i=ofx;i<dx;i++){for (int j=ofy;j<dy;j++){if (imTmp(j,i)!=0){im(j,i)=imTmp2(j,i)/(imTmp(j,i));}else{im(j,i)=0;}}}
		
	for (int i=ofx;i<dx;i+=factor){
		for (int j=ofy;j<dy;j+=factor){
			imSub((int)((float)(j-ofx)/(float)factor),(int)((float)(i-ofy)/(float)factor))=im(j,i);
		}
	}	
	imSub=imSub.crop(0,0,dxSub-((int)(ofx/factor))*2-1,dySub-((int)(ofy/factor))*2-1);	
	return imSub;
}

void FillChannel(CImg<float> &im_T , const  CImg<float> & im, int band){
	
	int dx=im.dimx();
	int dy=im.dimy();
				
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im_T(i,j,band)=im(i,j);
		}
	}
}
void FillChannel(CImg<double> &im_T , const  CImg<double> & im, int band){
	
	int dx=im.dimx();
	int dy=im.dimy();
				
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im_T(i,j,band)=im(i,j);
		}
	}
}
void FillChannel(CImg<double> &im_T , const  CImg<float> & im, int band){
	
	int dx=im.dimx();
	int dy=im.dimy();
				
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im_T(i,j,band)=im(i,j);
		}
	}
}

void isolateLayer(CImg<unsigned short> &im , const  CImg<unsigned char> & imClass, int classMin, int classMax){

	int dx=im.dimx();
	int dy=im.dimy();

	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
		     if(!((int)imClass(i,j)>=classMin && (int)imClass(i,j)<=classMax)){
		     	     if ((int)imClass(i,j)<=classMax){im(i,j)=1022;}
			     else{im(i,j)=1023;}
		     }  	     
		}
	}
} 
void addMotion2mozaic(CImg<float> &compos, const  CImg<float> & motion,  const  CImg<unsigned char> & imClass, int classMin, int classMax){
	
	int dx=compos.dimx();
	int dy=compos.dimy();
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
		     if(((int)imClass(i,j)>=classMin && (int)imClass(i,j)<=classMax)){
		     	     compos(i,j,0)=motion(i,j,0);
			     compos(i,j,1)=motion(i,j,1);
		     }  	     
		}
	}

}
CImg<float> MultiplyCImg(const CImg<float> &a,const CImg<float> &b){
	
	int dx=a.dimx();
	int dy=a.dimy();
	CImg<float> im(dx,dy);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	im(i,j)=a(i,j)*b(i,j);
	   }
	}
	return im;
}
CImg<double> MultiplyCImg(const CImg<double> &a,const CImg<double> &b){
	
	int dx=a.dimx();
	int dy=a.dimy();
	CImg<double> im(dx,dy);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	im(i,j)=a(i,j)*b(i,j);
	   }
	}
	return im;
}
CImg<double> MultiplyCImg(const CImg<float> &a,const CImg<double> &b){
	
	int dx=a.dimx();
	int dy=a.dimy();
	CImg<double> im(dx,dy);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	im(i,j)=a(i,j)*b(i,j);
	   }
	}
	return im;
}
CImg<double> DevideCImg(const CImg<double> &a,const CImg<double> &b){
	
	int dx=a.dimx();
	int dy=a.dimy();
	CImg<double> im(dx,dy);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	im(i,j)=a(i,j)/b(i,j);
	   }
	}
	return im;
}

	
CImg<float> grad_VieilleFaugeras(const CImg<float> &im, float pixelScale){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<float> Imx(dx,dy);
	CImg<float> Imy(dx,dy);
	CImg<float> grad(dx,dy,2);
	
	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=-135.0;	 filtre_GX2(0,1)=15.0;	 filtre_GX2(0,2)=65.0;  filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=-135.0;
		filtre_GX2(1,0)=188.0;	 filtre_GX2(1,1)=263.0;  filtre_GX2(1,2)=288.0; filtre_GX2(1,3)=263.0;   filtre_GX2(1,4)=188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0;   filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=-188.0;	 filtre_GX2(3,1)=-263.0; filtre_GX2(3,2)=-288.0;filtre_GX2(3,3)=-263.0;  filtre_GX2(3,4)=-188.0;
		filtre_GX2(4,0)=135.0;	 filtre_GX2(4,1)=-15.0;	 filtre_GX2(4,2)=-65.0; filtre_GX2(4,3)=-15.0;	 filtre_GX2(4,4)=135.0;
		
  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;
			
	Imx=im.get_convolve(filtre_GX2,1,false);		
	Imx*=normCst;
	Imy=im.get_convolve(filtre_GY2,1,false);
	Imy*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	grad(i,j,0)=Imx(i,j);
		grad(i,j,1)=Imy(i,j);
	   }
	}		
	return grad;
}

/*CImg<double> grad_Gaussian(const CImg<double> &im, float sigma, float pixelScale){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<double> Imx(dx,dy);
	CImg<double> Imy(dx,dy);
	CImg<double> grad(dx,dy,2);*/
	
	/*-- filtre gradiant gaussien --*/
	/*cout<<"Kernel support size :"<<(int)(sigma*4)+1<<endl;
	CImg<double> filtre_GX2((int)(sigma*4)+1,1);
		for(int i=0;i<(int)(sigma*2);i++){
			filtre_GX2(i,0)=-(double)(i-(int)(sigma*2))*exp(-pow((double)(i-(int)(sigma*2)),2)/(2.*pow(sigma,2)))/(2.*M_PI*pow(sigma,4));
			
		}
		filtre_GX2((int)(sigma*2),0)=0.0;
		for(int i=1;i<(int)(sigma*2)+1;i++){
			filtre_GX2((int)(sigma*2)+i,0)=-filtre_GX2((int)(sigma*2)-i,0);	
			
		}
	
  	CImg<double> filtre_GY2((int)(sigma*4)+1,1);
	filtre_GY2=filtre_GX2.get_transpose();
	
		
	double normCst=1.0/pixelScale;
			
	Imx=im.get_convolve(filtre_GX2,1,false);		
	Imx*=normCst;
	Imy=im.get_convolve(filtre_GY2,1,false);
	Imy*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	grad(i,j,0)=Imx(i,j);
		grad(i,j,1)=Imy(i,j);
	   }
	}	
	return grad;
}
CImg<double> grad_Gaussian(const CImg<float> &im, float sigma, float pixelScale){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<double> Imx(dx,dy);
	CImg<double> Imy(dx,dy);
	CImg<double> grad(dx,dy,2);
	
	//discrete kernel computation
	CImg<double> gaussDerivate(511);
	CImg<double> interpolator(511);
	CImg<double> kernel(511);
	CImg<double> filtre_GX2((int)(sigma*6)+1,1);
	
	float delta=(6.*sigma)/511.;
	for (int i=0;i<256;i++){
		gaussDerivate(i)=-(double)((i-255)*delta)*exp(-pow((double)((i-255)*delta),2)/(2.*pow(sigma,2)))/(2.*M_PI*pow(sigma,4));
	}
	for (int i=0;i<255;i++){interpolator(i)=(double)sin(-M_PI*(i-255)*6./511)/(-M_PI*(i-255)*6./511);}
	interpolator(255)=1.;
	for (int i=1;i<256;i++){
		interpolator(i+255)=interpolator(255-i);
		gaussDerivate(i+255)=-gaussDerivate(255-i);
	}
	kernel=gaussDerivate.get_convolve(interpolator,1,false);
	interpolator.display();gaussDerivate.display();kernel.display();
	for (int i=0;i<(int)(sigma*6)+1;i++){
		filtre_GX2(i,0)=kernel((int)(i/(delta)));
		cout<<filtre_GX2(i,0)<<"   "<<gaussDerivate((int)(i/(delta)))<<endl;
	}*/
	
	/*-- filtre gradiant gaussien --*/
	/*cout<<"Kernel support size :"<<(int)(sigma*4)+1<<endl;
	CImg<double> filtre_GX2((int)(sigma*4)+1,1);
		for(int i=0;i<(int)(sigma*2 );i++){
			filtre_GX2(i,0)=-(double)(i-(int)(sigma*2))*exp(-pow((double)(i-(int)(sigma*2)),2)/(2.*pow(sigma,2)))/(2.*M_PI*pow(sigma,4));
			
		}	
		filtre_GX2((int)(sigma*2),0)=0.0;
		for(int i=1;i<(int)(sigma*2)+1;i++){
			filtre_GX2((int)(sigma*2)+i,0)=-filtre_GX2((int)(sigma*2)-i,0);	
			
		}
	*/
/*  	CImg<double> filtre_GY2((int)(sigma*6)+1,1);
	filtre_GY2=filtre_GX2.get_transpose();
	
		
	double normCst=1.0/pixelScale;
			
	Imx=im.get_convolve(filtre_GX2,1,false);		
	Imx*=normCst;
	Imy=im.get_convolve(filtre_GY2,1,false);
	Imy*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	grad(i,j,0)=Imx(i,j);
		grad(i,j,1)=Imy(i,j);
	   }
	}	
	Imx.display();Imy.display();
	return grad;
}
CImg<double> laplacian_Gaussian(const CImg<double> &im,float  sigma, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	CImg<double> Imxx(dx,dy);
	CImg<double> Imyy(dx,dy);
	//CImg<double> Imxy(dx,dy);
	CImg<double> laplacian(dx,dy,3);*/
	
	/*-- filtre gradiant gaussien --*/
/*	cout<<"Kernel support size :"<<(int)(sigma*8)+1<<endl;
	CImg<double> filtre_GXX((int)(sigma*8)+1,1);
		for(int i=0;i<(int)(sigma*4);i++){
			filtre_GXX(i,0)=-(double)exp(-pow((double)(i-(int)(sigma*2)),2)/(2.*pow(sigma,2)))*(-pow((double)(i-(int)(sigma*2)),2)/(2.*M_PI*pow(sigma,6))-1./(2.*M_PI*pow(sigma,4)));
		cout<<filtre_GXX(i,0)<<endl;	
		}
		filtre_GXX((int)(sigma*4),0)=-1./(2.*M_PI*pow(sigma,4));cout<<filtre_GXX((int)(sigma*4),0)<<endl;
		for(int i=1;i<(int)(sigma*4)+1;i++){
			filtre_GXX((int)(sigma*4)+i,0)=filtre_GXX((int)(sigma*4)-i,0);	
			cout<<filtre_GXX((int)(sigma*4)+i,0)<<endl;
			
		}
	
  	CImg<double> filtre_GYY((int)(sigma*4)+1,1);
	filtre_GYY=filtre_GXX.get_transpose();
	
		
	double normCst=1.0/pixelScale;
			
	Imxx=im.get_convolve(filtre_GXX,1,false);		
	Imxx*=normCst*normCst; 
	Imyy=im.get_convolve(filtre_GYY,1,false);
	Imyy*=normCst*normCst;
	
	Imxx.display();grad_Gaussian(grad_Gaussian(im,sigma,pixelScale).get_slice(0),sigma,pixelScale).get_slice(0).display();
	
	Imyy.display();grad_Gaussian(grad_Gaussian(im,sigma,pixelScale).get_slice(1),sigma,pixelScale).get_slice(1).display();
	
	
	
	//Imxy=im.get_convolve(filtre_GXY,1,false);
	//Imxy*=normCst*normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	laplacian(i,j,0)=Imxx(i,j);
		laplacian(i,j,1)=Imyy(i,j);
		//laplacian(i,j,2)=Imxy(i,j); //A FINIR SI BESOIN DE DERIVEE CROISEE!!!!
	   }
	}	
	
	return laplacian;
	
}*/

float pixMax(const CImg<float> &im){
	
	int dx=im.dimx();
	int dy=im.dimy();
	float max=-1E10;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>max)
	   		max=im(i,j);
	   }
	}
	return max;
}
CImg<double> treshold(const CImg<double> &im, float min, float max){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<double> outIm(dx,dy);outIm.fill(0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>=min&&im(i,j)<=max)
	   		outIm(i,j)=im(i,j);
	   }
	}
	return outIm;
}
CImg<float> tresholdMin(const CImg<float> &im, float min){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<float> outIm(dx,dy);outIm.fill(0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>=min)
	   		outIm(i,j)=im(i,j);
	   }
	}
	return outIm;
}
CImg<unsigned char> treshold(const CImg<unsigned char> &im, int min, int max){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<unsigned char> outIm(dx,dy);
	outIm.fill(0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>=min&&im(i,j)<=max)
	   		outIm(i,j)=im(i,j);
	   }
	}
	return outIm;
}
CImg<float> treshold(const CImg<float> &im, int min, int max){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<float> outIm(dx,dy);
	outIm.fill(0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>=min&&im(i,j)<=max)
	   		outIm(i,j)=im(i,j);
	   }
	}
	return outIm;
}

CImg<unsigned char> treshold_bin(const CImg<> &im, int min, int max){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<> outIm(dx,dy);
	outIm.fill(0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>=min&&im(i,j)<=max)
	   		outIm(i,j)=1;
	   }
	}
	return outIm;
}


CImg<unsigned char> tresholdBin(const CImg<double> &im, float min, float max){
	
	int dx=im.dimx();
	int dy=im.dimy();
	CImg<unsigned char> outIm(dx,dy);
	outIm.fill(0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if(im(i,j)>=min&&im(i,j)<=max)
	   		outIm(i,j)=1;
	   }
	}
	return outIm;
}

CImg<double> hessian(const CImg<double> &grad){
	
	int dx=grad.dimx();
	int dy=grad.dimy();
	
	CImg<double> hessian(dx,dy,2,2);
	
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	hessian(i,j,0,0)=grad(i,j,0)*grad(i,j,0);
		hessian(i,j,1,1)=grad(i,j,1)*grad(i,j,1);
		hessian(i,j,0,1)=grad(i,j,0)*grad(i,j,1);
		hessian(i,j,1,0)=grad(i,j,0)*grad(i,j,1);
	   }
	}
	return hessian;
}

float MeanImage(const CImg<float> &im){
	
	int dx=im.dimx();
	int dy=im.dimy();
	
	float res=0;
	
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	res+=im(i,j);
	   }
	}
	res/=(dx*dy);
	return res;
}

float MeanImageMasked(const CImg<float> &im,const CImg<int> &mask){
	
	int dx=im.dimx();
	int dy=im.dimy();
	
	float res=0;
	long int nbPix=0;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	   	if (mask(i,j)!=0){	
	   		res+=im(i,j);
			nbPix++;
		}
	   }
	}
	res/=nbPix;
	return res;
}


CImg<float> gradientXYMultiBand(const CImg<float> &im, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	int nbLayer=im.dimz();
		
	CImg <float> gradMulti(dx,dy,2,nbLayer);
	
	CImg <float> aux(dx,dy);
	CImgl <float> auxGrad(2,dx,dy);
	
	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=135.0;	 filtre_GX2(0,1)=-15.0;	 filtre_GX2(0,2)=-65.0; filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=135.0;
		filtre_GX2(1,0)=-188.0;	 filtre_GX2(1,1)=-263.0; filtre_GX2(1,2)=-288.0; filtre_GX2(1,3)=-263.0; filtre_GX2(1,4)=-188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0; filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=188.0;	 filtre_GX2(3,1)=263.0;	 filtre_GX2(3,2)=288.0; filtre_GX2(3,3)=263.0;	 filtre_GX2(3,4)=188.0;
		filtre_GX2(4,0)=-135.0;	 filtre_GX2(4,1)=15.0;	 filtre_GX2(4,2)=65.0; filtre_GX2(4,3)=15.0;	 filtre_GX2(4,4)=-135.0;
		
  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;

	for (int k=0;k<nbLayer;k++) {
		aux=im.get_slice(k);
		auxGrad[0]=aux.get_convolve(filtre_GX2,1,false);		
		auxGrad[0]*=normCst;
		auxGrad[1]=aux.get_convolve(filtre_GY2,1,false);		
		auxGrad[1]*=normCst;
		for (int i=0;i<dx;i++){
		   for (int j=0;j<dy;j++){
			gradMulti(i,j,0,k)=auxGrad[0](i,j);
			gradMulti(i,j,1,k)=auxGrad[1](i,j);
		   }
	        }
	}	
	return gradMulti;
}	

CImg<float> divMultiBand(const CImg<float> &im, float pixelScale){
	
	int dx=im.dimx();
	int dy=im.dimy();
	int nbLayer=im.dimv();
	
	CImg<float> divIm(dx,dy,nbLayer);
	CImg<float> Ux(dx,dy);CImg<float> Vy(dx,dy);
	CImg<float> u(dx,dy);
	CImg<float> v(dx,dy);	

	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=-135.0;	 filtre_GX2(0,1)=15.0;	 filtre_GX2(0,2)=65.0;  filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=-135.0;
		filtre_GX2(1,0)=188.0;	 filtre_GX2(1,1)=263.0;  filtre_GX2(1,2)=288.0; filtre_GX2(1,3)=263.0;   filtre_GX2(1,4)=188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0;   filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=-188.0;	 filtre_GX2(3,1)=-263.0; filtre_GX2(3,2)=-288.0;filtre_GX2(3,3)=-263.0;  filtre_GX2(3,4)=-188.0;
		filtre_GX2(4,0)=135.0;	 filtre_GX2(4,1)=-15.0;	 filtre_GX2(4,2)=-65.0;  filtre_GX2(4,3)=-15.0;	 filtre_GX2(4,4)=135.0;
		
  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;
			
	for (int k=1;k<nbLayer;k++) {
		u=(im.get_channel(k)).get_slice(0);
		v=(im.get_channel(k)).get_slice(1);
		Ux=u.get_convolve(filtre_GX2,1,false);		
		Ux*=normCst;
		Vy=v.get_convolve(filtre_GY2,1,false);
		Vy*=normCst;
		for (int i=0;i<dx;i++){
		   for (int j=0;j<dy;j++){	
		   	divIm(i,j,k)=Ux(i,j)+Vy(i,j);
		   }
		}
		//divIm.get_slice(k).display();
		
		
	}
	return divIm;
}


CImg<float> divIm(const CImg<float> &im, float pixelScale){
	
	int dx=im.dimx();
	int dy=im.dimy();
	
	CImg<float> divIm(dx,dy);
	CImg<float> Ux(dx,dy);CImg<float> Vy(dx,dy);
	CImg<float> u(dx,dy);
	CImg<float> v(dx,dy);	


	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=-135.0;	 filtre_GX2(0,1)=15.0;	 filtre_GX2(0,2)=65.0;  filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=-135.0;
		filtre_GX2(1,0)=188.0;	 filtre_GX2(1,1)=263.0;  filtre_GX2(1,2)=288.0; filtre_GX2(1,3)=263.0;   filtre_GX2(1,4)=188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0;   filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=-188.0;	 filtre_GX2(3,1)=-263.0; filtre_GX2(3,2)=-288.0;filtre_GX2(3,3)=-263.0;  filtre_GX2(3,4)=-188.0;
		filtre_GX2(4,0)=135.0;	 filtre_GX2(4,1)=-15.0;	 filtre_GX2(4,2)=-65.0;  filtre_GX2(4,3)=-15.0;	 filtre_GX2(4,4)=135.0;

  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;
			
	
	u=im.get_slice(0);
	v=im.get_slice(1);
	Ux=u.get_convolve(filtre_GX2,1,false);		
	Ux*=normCst;
	Vy=v.get_convolve(filtre_GY2,1,false);
	Vy*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	divIm(i,j)=Ux(i,j)+Vy(i,j);
	   }
	}
	//divIm.get_slice(k).display();
		
		
	
	return divIm;
}
CImg<double> divImOrder2(const CImg<double> &im, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	
	CImg<double> divIm(dx,dy);
	CImg<float> Ux(dx,dy);CImg<float> Vy(dx,dy);
	CImg<float> u(dx,dy);
	CImg<float> v(dx,dy);	

	/*-- filtre gradiant --*/
	CImg<float> filtre_GX2(3,1);
		
		filtre_GX2(0,0)=1.0;
		filtre_GX2(1,0)=0.0;
		filtre_GX2(2,0)=-1.0;
		
	 
  	CImg<float> filtre_GY2(1,3);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=0.5/pixelScale;
	
			
	u=im.get_slice(0);
	v=im.get_slice(1);
	Ux=u.get_convolve(filtre_GX2,1,false);		
	Ux*=normCst;
	Vy=v.get_convolve(filtre_GY2,1,false);
	Vy*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	divIm(i,j)=Vy(i,j)+Ux(i,j);
	   }
	}	
	
	return divIm;
}
CImg<float> curlMultiBand(const CImg<float> &im, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	int nbLayer=im.dimv();
	
	CImg<float> divIm(dx,dy,nbLayer);
	CImg<float> Uy(dx,dy);CImg<float> Vx(dx,dy);
	CImg<float> u(dx,dy);
	CImg<float> v(dx,dy);	

	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=-135.0;	 filtre_GX2(0,1)=15.0;	 filtre_GX2(0,2)=65.0;  filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=-135.0;
		filtre_GX2(1,0)=188.0;	 filtre_GX2(1,1)=263.0;  filtre_GX2(1,2)=288.0; filtre_GX2(1,3)=263.0;   filtre_GX2(1,4)=188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0;   filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=-188.0;	 filtre_GX2(3,1)=-263.0; filtre_GX2(3,2)=-288.0;filtre_GX2(3,3)=-263.0;  filtre_GX2(3,4)=-188.0;
		filtre_GX2(4,0)=135.0;	 filtre_GX2(4,1)=-15.0;	 filtre_GX2(4,2)=-65.0;  filtre_GX2(4,3)=-15.0;	 filtre_GX2(4,4)=135.0;
	//NB: filtre_GX2(col,line)
	
  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;
			
	for (int k=1;k<nbLayer;k++) {
		u=(im.get_channel(k)).get_slice(0);
		v=(im.get_channel(k)).get_slice(1);
		Uy=u.get_convolve(filtre_GY2,1,false);		
		Uy*=normCst;
		Vx=v.get_convolve(filtre_GX2,1,false);
		Vx*=normCst;
		for (int i=0;i<dx;i++){
		   for (int j=0;j<dy;j++){	
		   	divIm(i,j,k)=Vx(i,j)-Uy(i,j);
		   }
		}
		//divIm.get_slice(k).display();
		
		
	}
	return divIm;
}

CImg<double> curlImOrder2(const CImg<double> &im, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	
	CImg<double> divIm(dx,dy);
	CImg<float> Uy(dx,dy);CImg<float> Vx(dx,dy);
	CImg<float> u(dx,dy);
	CImg<float> v(dx,dy);	

	/*-- filtre gradiant --*/
	CImg<float> filtre_GX2(3,1);
		
		filtre_GX2(0,0)=1.0;
		filtre_GX2(1,0)=0.0;
		filtre_GX2(2,0)=-1.0;
		
	 
  	CImg<float> filtre_GY2(1,3);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=0.5/pixelScale;
	
			
	u=im.get_slice(0);
	v=im.get_slice(1);
	Uy=u.get_convolve(filtre_GY2,1,false);		
	Uy*=normCst;
	Vx=v.get_convolve(filtre_GX2,1,false);
	Vx*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	divIm(i,j)=Vx(i,j)-Uy(i,j);
	   }
	}	
	
	return divIm;
}

CImg<double> curlIm(const CImg<double> &im, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	
	CImg<double> divIm(dx,dy);
	CImg<float> Uy(dx,dy);CImg<float> Vx(dx,dy);
	CImg<float> u(dx,dy);
	CImg<float> v(dx,dy);	

	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=-135.0;	 filtre_GX2(0,1)=15.0;	 filtre_GX2(0,2)=65.0;  filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=-135.0;
		filtre_GX2(1,0)=188.0;	 filtre_GX2(1,1)=263.0;  filtre_GX2(1,2)=288.0; filtre_GX2(1,3)=263.0;   filtre_GX2(1,4)=188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0;   filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=-188.0;	 filtre_GX2(3,1)=-263.0; filtre_GX2(3,2)=-288.0;filtre_GX2(3,3)=-263.0;  filtre_GX2(3,4)=-188.0;
		filtre_GX2(4,0)=135.0;	 filtre_GX2(4,1)=-15.0;	 filtre_GX2(4,2)=-65.0;  filtre_GX2(4,3)=-15.0;	 filtre_GX2(4,4)=135.0;
	 
  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;
	
			
	u=im.get_slice(0);
	v=im.get_slice(1);
	Uy=u.get_convolve(filtre_GY2,1,false);		
	Uy*=normCst;
	Vx=v.get_convolve(filtre_GX2,1,false);
	Vx*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){	
	   	divIm(i,j)=Vx(i,j)-Uy(i,j);
	   }
	}	
	
	return divIm;
	//return divIm.crop(2,2,2+dx-1,2+dy-1,false);
}
CImg<double> curlIm_dx2(const CImg<double> &im, float pixelScale){

	int dx=im.dimx();
	int dy=im.dimy();
	
	CImg<double> divIm(dx,dy*2);
	CImg<float> Uy(dx,dy*2);CImg<float> Vx(dx,dy*2);
	CImg<float> u(dx,dy*2);
	CImg<float> v(dx,dy*2);	

	/*-- filtre gradiant: Vieville Faugeras ECCV'92 --*/
	CImg<float> filtre_GX2(5,5);
		filtre_GX2(0,0)=-135.0;	 filtre_GX2(0,1)=15.0;	 filtre_GX2(0,2)=65.0;  filtre_GX2(0,3)=15.0;	 filtre_GX2(0,4)=-135.0;
		filtre_GX2(1,0)=188.0;	 filtre_GX2(1,1)=263.0;  filtre_GX2(1,2)=288.0; filtre_GX2(1,3)=263.0;   filtre_GX2(1,4)=188.0;
		filtre_GX2(2,0)=0.0;	 filtre_GX2(2,1)=0.0;	 filtre_GX2(2,2)=0.0;   filtre_GX2(2,3)=0.0;	 filtre_GX2(2,4)=0.0;
		filtre_GX2(3,0)=-188.0;	 filtre_GX2(3,1)=-263.0; filtre_GX2(3,2)=-288.0;filtre_GX2(3,3)=-263.0;  filtre_GX2(3,4)=-188.0;
		filtre_GX2(4,0)=135.0;	 filtre_GX2(4,1)=-15.0;	 filtre_GX2(4,2)=-65.0;  filtre_GX2(4,3)=-15.0;	 filtre_GX2(4,4)=135.0;
	 
  	CImg<float> filtre_GY2(5,5);
		filtre_GY2=filtre_GX2.get_transpose();
		
	float normCst=1.0/1680.0/pixelScale;
	
			
	u=(im.get_slice(0)).get_resize(dx,dy*2,3);
	v=(im.get_slice(1)).get_resize(dx,dy*2,3);
	Uy=u.get_convolve(filtre_GY2,1,false);		
	Uy*=normCst;
	Vx=v.get_convolve(filtre_GX2,1,false);
	Vx*=normCst;
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy*2;j++){	
	   	divIm(i,j)=Vx(i,j)-Uy(i,j);
	   }
	}	
	
	return divIm;
	//return divIm.crop(2,2,2+dx-1,2+dy-1,false);
}

void eigenDecomposition(const CImg<double> &A, CImg<double> &eigenVal, CImg<double> &eigenVec){

	int dx=A.dimx();
	int dy=A.dimy();
	CImg <double> aux(2,2);
	CImg<double> eigenValAux(2);
	CImg<double> eigenVecAux(2,2);
	
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	 	  aux(0,0)=A(i,j,0,0);aux(1,0)=A(i,j,1,0);
		  aux(0,1)=A(i,j,0,1);aux(1,1)=A(i,j,1,1);  
		  aux.symeigen(eigenValAux,eigenVecAux);
		  eigenVal(i,j,0)=eigenValAux(0);
		  eigenVal(i,j,1)=eigenValAux(1);
		  eigenVec(i,j,0,0)=eigenVecAux(0,0);
		  eigenVec(i,j,0,1)=eigenVecAux(0,1); 
		  eigenVec(i,j,1,0)=eigenVecAux(1,0);
		  eigenVec(i,j,1,1)=eigenVecAux(1,1);
	   }
	}

}
CImg<unsigned char> Num_Rank(const CImg<double> &eigenVal,double eta,double eps){
        
	int dx=eigenVal.dimx();
	int dy=eigenVal.dimy();
	CImg<unsigned char> RankTensor(dx,dy);
	
	RankTensor=tresholdBin(MultiplyCImg(eigenVal.get_slice(0),eigenVal.get_slice(0))+MultiplyCImg(eigenVal.get_slice(1),eigenVal.get_slice(1)),eta*eta,1e10);
	
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	 	if(RankTensor(i,j)>0){
			if( eigenVal(i,j,1)==0){RankTensor(i,j)=1;}
			else{
				if((log(eigenVal(i,j,0)/eigenVal(i,j,1))/log(2.))>=eps){RankTensor(i,j)=1;}
				else{RankTensor(i,j)=2;}
			}
		}
	   }
	}	
	RankTensor.display();
	return RankTensor;
}

CImg<double> lucasKanade(const CImg<float> &BMatrix,const CImg<double> &eigenVal,const CImg<double> &eigenVec,const CImg<unsigned char> &RankTensor){
	
	int dx=eigenVal.dimx();
	int dy=eigenVal.dimy();
	CImg<double> Uloc(dx,dy,2);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
		if(RankTensor(i,j)==2){
			Uloc(i,j,0)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,0)+BMatrix(i,j,1)/eigenVal(i,j,1)*eigenVec(i,j,1,0);
			Uloc(i,j,1)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,1)+BMatrix(i,j,1)/eigenVal(i,j,1)*eigenVec(i,j,1,1);
		}
		if(RankTensor(i,j)==1){
			Uloc(i,j,0)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,0);
			Uloc(i,j,1)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,1);
		}
		if(RankTensor(i,j)==0){
			Uloc(i,j,0)=0.0;Uloc(i,j,1)=0.0;
		}
	   }
	}
		
	return Uloc;
}
CImg<double> lucasKanadeNorm(const CImg<double> &BMatrix,const CImg<double> &eigenVal,const CImg<double> &eigenVec,const CImg<unsigned char> &RankTensor){
	
	int dx=eigenVal.dimx();
	int dy=eigenVal.dimy();
	CImg<double> Uloc(dx,dy,2,2);
	Uloc.fill(0.0);
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
		if(RankTensor(i,j)==2){
			//normal vector
			Uloc(i,j,0,0)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,0);
			Uloc(i,j,0,1)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,1);
			//tangential vector
			Uloc(i,j,1,0)=BMatrix(i,j,1)/eigenVal(i,j,1)*eigenVec(i,j,1,0);
			Uloc(i,j,1,1)=BMatrix(i,j,1)/eigenVal(i,j,1)*eigenVec(i,j,1,1);
		}
		if(RankTensor(i,j)==1){
			//normal vector
			Uloc(i,j,0,0)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,0);
			Uloc(i,j,0,1)=BMatrix(i,j,0)/eigenVal(i,j,0)*eigenVec(i,j,0,1);
		}
		
	   }
	}
		
	return Uloc;
}
CImg<int > erodeBinary(const CImg<unsigned char> &imMask,int classMin, int classMax, int sizeElement){
	
	int dx=imMask.dimx();
	int dy=imMask.dimy();
	int Id,Ig,Jd,Jg;		//border image auxiliary index		
	bool erode,inlayer, check;
	
	CImg<int> erodeMask(dx,dy);
	CImg<int> res(dx,dy);
	
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			inlayer=(int)imMask(i,j)>=classMin && (int)imMask(i,j)<=classMax;
			if(inlayer){erodeMask(i,j)=(int)imMask(i,j);}
			else{erodeMask(i,j)=-1;}
		}
	}
	res.operator=(erodeMask);
	sizeElement=(int)((float)sizeElement/2.);
	
	//Erosion  	
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
		   for (int k1=1;k1<=sizeElement;k1++){
		     for (int k2=1;k2<=sizeElement;k2++){
			Id=i+k1;Ig=i-k1; 
			Jd=j+k2;Jg=j-k2;
			if(Id>dx-1){Id=dx-1;Ig=2*i-dx+1;}//(image border -> decreasing size of structurant element)
			if(Ig<0){Ig=2*i;}
			if(Jd>dy-1){Jd=dy-1;Jg=2*j-dy+1;}
			if(Jg<0){Jg=2*j;}	
			if (erodeMask(i,j)>0){
				erode=erodeMask(Ig,Jg)<0 || erodeMask(Ig,j)<0 || erodeMask(Ig,Jd)<0;
				erode=erode || erodeMask(i,Jg)<0 || erodeMask(i,Jd)<0;
				erode=erode || erodeMask(Id,Jg)<0 || erodeMask(Id,j)<0 || erodeMask(Id,Jd)<0;
				if (erode){res(i,j)=-1;}
			}
		     }
		   }
		}
	}
	check=false;
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
		 	if(res(i,j)>0){check=true;}
		}
	}
	if (check==false){cout<<"Error : empty mask -> erosion cancelled"<<endl;res=imMask;}
	return res;
} 

bool * FourConnexNeighbours(int i, int j,const CImg<bool> &contour){
	int dx=contour.dimx();
	int dy=contour.dimy();
	int Id,Ig,Jd,Jg;
	bool * connex=new bool[4];
	 	Id=i+1;Ig=i-1; //(image border -> mirror)
	    	if(i==dx-1){Id=dx-2;}
	    	if(i==0){Ig=1;}
	    	Jd=j+1;Jg=j-1;
	    	if(j==dy-1){Jd=dy-2;}
	    	if(j==0){Jg=1;}   
		
		connex[0]=contour(Id,j);
		connex[1]=contour(Ig,j);
		connex[2]=contour(i,Jd);
		connex[3]=contour(i,Jg);
				
	return connex;
}
bool  aSharedNeighbour(bool *a, bool * b, int length){
	
	bool res=false;
	for (int i=0;i<length;i++){
		if(a[i]&&b[i]){res=true; break;}
	}
	return res;
}

CImg<short> copyVecCImg(short *image, int dx,int dy){
	
	CImg<short> im(dx,dy);
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j)=image[j*dx+i];
		}
	}
	return im;
}

CImg<double> copyVecCImg(float *image, int dx,int dy){
	
	CImg<double> im(dx,dy);
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j)=image[j*dx+i];
		}
	}
	return im;
}
CImg<float> copyVecSCImgF(short *image, int dx,int dy){
	
	CImg<float> im(dx,dy);
	for (int j=0;j<dy;j++){
		for (int i=0;i<dx;i++){
			im(i,j)=image[j*dx+i];
		}
	}
	return im;
}

void VecFCImgF(CImg<float> &im, float *image){
	
	int dx=im.dimx();
	int dy=im.dimy();
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j)=image[j*dx+i];
		}
	}

}

void CImgFVecF(float *image,const CImg<float> &im){
	int dx=im.dimx();
	int dy=im.dimy();
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			image[j*dx+i]=im(i,j);
		}
	}
	
}

CImg<float> copyVecFCImgF(float *image, int dx,int dy){
	
	CImg<float> im(dx,dy);
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j)=image[j*dx+i];
		}
	}
	return im;
}

float * copyCImgVec(const CImg<double> &X, int dx, int dy){

	float* im=new float[(unsigned int)dx*(unsigned int)dy];
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im[j*dx+i]=X(i,j);
		}
	}
	return im;

}
float * copyCImgVec(const CImg<float> &X, int dx, int dy){

	float* im=new float[(long int)dx*(long int)dy];
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im[j*dx+i]=X(i,j);
		}
	}
	return im;

}
float * copyCImgVec(const CImg<float> &X){

        int dx=X.dimx();
	int dy=X.dimy();
	float* im=new float[(long int)dx*(long int)dy];
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im[j*dx+i]=X(i,j);
		}
	}
	return im;

}
float * copyCImgVec(const CImg<short> &X, int dx, int dy){

	float* im=new float[(long int)dx*(long int)dy];
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im[j*dx+i]=X(i,j);
		}
	}
	return im;

}


CImg<double> MultiSlice(const CImg<double> &U,const CImg<double> &V){
	
	int dx=U.dimx();
	int dy=U.dimy();
	CImg<double> im(dx,dy,2);
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j,0)=U(i,j);
			im(i,j,1)=V(i,j);
		}
	}
	return im;
}

CImg<float> MultiSliceF(const CImg<double> &U,const CImg<double> &V){
	
	int dx=U.dimx();
	int dy=U.dimy();
	CImg<float> im(dx,dy,2);
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j,0)=U(i,j);
			im(i,j,1)=V(i,j);
		}
	}
	return im;
}
CImg<float> MultiSliceF(const CImg<float> &U,const CImg<float> &V){
	
	int dx=U.dimx();
	int dy=U.dimy();
	CImg<float> im(dx,dy,2);
	for (int i=0;i<dx;i++){
		for (int j=0;j<dy;j++){
			im(i,j,0)=U(i,j);
			im(i,j,1)=V(i,j);
		}
	}
	return im;
}


CImg<unsigned int > orientedCountour(const CImg<unsigned char> &imMask){
	
	int dx=imMask.dimx();
	int dy=imMask.dimy();
	int Id,Ig,Jd,Jg;		//border image auxiliary index
	CImg<int> norm(dx,dy,2);
	CImg<bool> contour(dx,dy);	//class contour 
	CImg<unsigned int> dContour(dx,dy);	//directed contour
	bool inlayer, bound;
	imMask.display();
	
	//defining the contour
	for (int i=0;i<dx;i++){
	    for (int j=0;j<dy;j++){
	    	    inlayer=imMask(i,j)<255;
	    	    bound=false;    
	    	    if(inlayer){
	    		    Id=i+1;Ig=i-1; //(image border -> mirror)
	    		    if(i==dx-1){Id=dx-2;}
	    		    if(i==0){Ig=1;}
	    		    Jd=j+1;Jg=j-1;
	    		    if(j==dy-1){Jd=dy-2;}
	    		    if(j==0){Jg=1;}   
	    		    bound=(!(imMask(Id,j)<255)||!(imMask(Ig,j)<255)||!(imMask(i,Jd)<255)||!(imMask(i,Jg)<255)); //4 convexite
	    	    }
	    	    if(bound){contour(i,j)=true;}
		    else{contour(i,j)=false;}

	    }
	}
	contour.display();
	
	//defining the contour orientation
	dContour.fill(0);
	unsigned int k=0;
	int ii,jj;
	bool extendC;
	for (int i=0;i<dx;i++){
	    for (int j=0;j<dy;j++){	        
	    	if (contour(i,j)&&dContour(i,j)==0){
			
			Id=i+1;Ig=i-1; //(image border -> mirror)
	    		if(i==dx-1){Id=dx-2;}
	    		if(i==0){Ig=1;}
	    		Jd=j+1;Jg=j-1;
	    		if(j==dy-1){Jd=dy-2;}
	    		if(j==0){Jg=1;}   
			
			dContour(i,j)=k;
			k++;
			
			ii=i;
			jj=j;
			extendC=true;
			while(extendC==true){
			   extendC=false;
			   if(contour(Id,jj)&&dContour(Id,jj)==0&& aSharedNeighbour(FourConnexNeighbours(Id,jj,contour),FourConnexNeighbours(ii,jj,contour),4)){ 
			      ii=Id;dContour(ii,jj)=k;k++;extendC=true;
			   }
			   else{
			      if(contour(Ig,jj)&&dContour(Ig,jj)==0&& aSharedNeighbour(FourConnexNeighbours(Ig,jj,contour),FourConnexNeighbours(ii,jj,contour),4)){
			         ii=Ig;dContour(ii,jj)=k;k++;extendC=true;
			      }
			      else{
			         if(contour(ii,Jd)&&dContour(ii,Jd)==0&& aSharedNeighbour(FourConnexNeighbours(ii,Jd,contour),FourConnexNeighbours(ii,jj,contour),4)){
				    jj=Jd;dContour(ii,jj)=k;k++;extendC=true;
				 }
				 else{
			            if(contour(ii,Jg)&&dContour(ii,Jg)==0&& aSharedNeighbour(FourConnexNeighbours(ii,Jg,contour),FourConnexNeighbours(ii,jj,contour),4)){
				       jj=Jg;dContour(ii,jj)=k;k++;;extendC=true;
				    }
				    else{extendC=false;}
			         }
			      }
			   }
			   Id=ii+1;Ig=ii-1; //(image border -> mirror)
	    		   if(ii==dx-1){Id=dx-2;}
	    		   if(ii==0){Ig=1;}
	    		   Jd=jj+1;Jg=jj-1;
	    		   if(jj==dy-1){Jd=dy-2;}
	    		   if(jj==0){Jg=1;}   
			   	
			}//end while
		}
	    }
	}

	return dContour;
} 
float ** allocMatrix(int nbLines, int nbColumns){
	
	  float **x=new float*[nbLines];
	  for (int k=0;k<nbLines;k++)
		x[k]=new float[nbColumns];
	  return x;
}
float * allocVector(int nbLines){
	
	  float *x=new float[nbLines];
	  return x;
}

float  get_median(float *vec, int dim){
	
	float aux;
	float res;
	bool again=true;
	while (again){
	   again=false;
	   for (int i=0; i<dim-1;i++){
		if (vec[i]>vec[i+1]){
			aux=vec[i+1];
			vec[i+1]=vec[i];
			vec[i]=aux;
			again=true;
		}
	   }
	}	
	res=vec[int(dim/2)]; 
	return res;
}

CImg<float > Median(const CImg<float> im, int size){

	int dx=im.dimx();
	int dy=im.dimy();
	int I,J;		
	
	CImg<float> res(dx,dy);
	float *vector=new float[(int)(size*size)];
		
	int halfSize=(int)size/2;
		 	
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	      for (int k1=-halfSize;k1<=halfSize;k1++){
	   	for (int k2=-halfSize;k2<=halfSize;k2++){
	   	   I=i+k1;
	   	   J=j+k2;	   	   
	   	   if(I>dx-1){I=dx-1;}
	   	   if(I<0){I=0;}
	   	   if(J>dy-1){J=dy-1;}
	   	   if(J<0){J=0;} 					   
	   	   vector[(k1+halfSize)*size+k2+halfSize]=im(I,J);
	   	}
	      }
	      res(i,j)=get_median(vector,size*size);
	   }
	}
	delete [] vector;
	
	return res;
} 
CImg<float > MedianOnGrad(const CImg<float> im, int size, float tresh){

	int dx=im.dimx();
	int dy=im.dimy();
	int I,J;
	int Id,Jd,Ig,Jg;		
  	CImgl<float> grad(2,dx,dy);
  	CImg<float> gradNorm(dx,dy);   
  	grad=im.get_gradientXY(); 
  	gradNorm=(grad[0].mul(grad[0])+grad[1].mul(grad[1])).sqrt();
	
	CImg<float> res(dx,dy);
	float *vector=new float[(int)(size*size)];
		
	int halfSize=(int)size/2;
		 	
	for (int i=0;i<dx;i++){
	   for (int j=0;j<dy;j++){
	    Id=i+1;Ig=i-1;
	    Jd=j+1;Jg=j-1;
	    if(Jd>dy-1){Jd=dy-1;}
	    if(Jg<0){Jg=0;}
	    if(Id>dx-1){Id=dx-1;} 
	    if(Ig<0){Ig=0;}
	    if(gradNorm(Id,Jd)>tresh
	      ||gradNorm(Id,j)>tresh
	      ||gradNorm(Id,Jg)>tresh
	      ||gradNorm(i,Jd)>tresh
	      ||gradNorm(i,j)>tresh
	      ||gradNorm(i,Jg)>tresh
	      ||gradNorm(Ig,Jd)>tresh
	      ||gradNorm(Ig,j)>tresh
	      ||gradNorm(Ig,Jg)>tresh){
	      for (int k1=-halfSize;k1<=halfSize;k1++){
	   	for (int k2=-halfSize;k2<=halfSize;k2++){
	   	   I=i+k1;
	   	   J=j+k2;	   	   
	   	   if(I>dx-1){I=dx-1;}
	   	   if(I<0){I=0;}
	   	   if(J>dy-1){J=dy-1;}
	   	   if(J<0){J=0;} 					   
	   	   vector[(k1+halfSize)*size+k2+halfSize]=im(I,J);
	   	}
	      }
	      res(i,j)=get_median(vector,size*size);
	    }
	    else{res(i,j)=im(i,j);}
	   }
	}
	delete [] vector;
	
	return res;
} 


#endif
